// A program to display a simple cube with each face/vertex a different colour.
// Cube can be rotated by Trackball.
//
// Use 'f' to use face coloures
// Use 'v' to use vertex coloures (=> RGB colour cube)

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include "Trackball.h"
#include <iostream>
using namespace std;

const int windowWidth=400;
const int windowHeight=400;

// define vertices and edges of the house
const int numVertices=8;
const int numFaces=6;
const int numFaceVertices=4;
const float vertices[numVertices][3] =  {{0,0,0},{1,0,0},
	        {0,1,0},{1,1,0},{0,0,1},{1,0,1},{0,1,1},{1,1,1}};
const int faces[numFaces][numFaceVertices] = {
		{0,1,5,4},  // Bottom
        {1,3,7,5},  // Right
        {0,4,6,2},  // Left
        {2,6,7,3},  // Top
        {4,5,7,6},  // Front
        {0,2,3,1}}; // Back
const float faceColours[numFaces][3] =      // (R,G,B) colours of each face
	 {{1,0,0},{0,1,0},{0,0,1},{1,1,0},{1,0,1},{0,1,1}};
CTrackball trackball;

typedef enum {FACE_COLOURS, VERTEX_COLOURS} RENDER_OPTION;
RENDER_OPTION current_render_option=FACE_COLOURS;

void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'f': current_render_option=FACE_COLOURS; 
					glutPostRedisplay(); break;
		case 'v': current_render_option=VERTEX_COLOURS; 
					glutPostRedisplay(); break;
		default: trackball.tbKeyboard(key);
	}
}

void display(void)
{
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
    gluLookAt(0,0,4, 0,0,0, 0,1,0); // camera is on the z-axis
	trackball.tbMatrix();			// rotate the cube using the trackball ...
	glTranslatef(-0.5f,-0.5f,-0.5f);// ... and move it the the centre

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 	// clear colour buffer and depth buffer
	for (int i=0; i < numFaces; i++) 
	{
		const int* face = faces[i];	
		if (current_render_option==FACE_COLOURS)
		{
			glBegin(GL_POLYGON);
			glColor3fv(faceColours[i]);
			for (int vIndex=0; vIndex<numFaceVertices; vIndex++)
				glVertex3fv(vertices[face[vIndex]]);
			glEnd();
		}
		else if (current_render_option==VERTEX_COLOURS)
		{
			glBegin(GL_POLYGON);
			for (int vIndex=0; vIndex<numFaceVertices; vIndex++)
			{
				glColor3fv(vertices[face[vIndex]]);
				glVertex3fv(vertices[face[vIndex]]);
			}
			glEnd();
		}
		else
			cerr << "drawColourCube: unknown render option";
    }
	glFlush ();
	glutSwapBuffers();
}

void init(void) 
{
	// select clearing color (for glClear)
	glClearColor (1,1,1,1);	// RGB-value for white
	// enable depth buffering
	glEnable(GL_DEPTH_TEST);
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(33,1,2,8);
	trackball.tbInit(GLUT_LEFT_BUTTON);
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a double buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);		
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(windowWidth, windowHeight); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Coloured Cube");
	init ();								// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
